/**
 * color.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"

#include "attr.h"
#include "color.h"

static void
xlib_free_color(color)
	t_color *color;
{
	XFree(color);
}

static VALUE
xlib_color_new(argc, argv, class)
	int argc;
	VALUE *argv;
	VALUE class;
{
	XColor col;
	t_color *p;
	VALUE obj, red, green, blue;
	int cnt, n;
	
	obj = Data_Make_Struct(class, t_color, 0, xlib_free_color, p);
	cnt = rb_scan_args(argc, argv, "03", &red, &green, &blue);
	switch (cnt) {
	case 1:
		n = NUM2INT(red);
		col.blue  = (n & 0xff) << 8; n = n >> 8;
		col.green = (n & 0xff) << 8; n = n >> 8;
		col.red   = (n & 0xff) << 8; n = n >> 8;
		break;
	case 3:
		col.red   = (NUM2INT(red  ) & 0xff) << 8;
		col.green = (NUM2INT(green) & 0xff) << 8;
		col.blue  = (NUM2INT(blue ) & 0xff) << 8;
		break;
	default:
		col.pixel = 0;
		col.red   = 0;
		col.green = 0;
		col.blue  = 0;
		break;
	}
	col.flags = DoGreen | DoRed | DoBlue;
	*p = col;
	
	rb_obj_call_init(obj, 0, NULL);
	
	return obj;
}

DefineNumericAttr(color, pixel)
DefineNumericAttr(color, red)
DefineNumericAttr(color, green)
DefineNumericAttr(color, blue)

static VALUE
xlib_color_to_s(obj)
	VALUE obj;
{
	char *s;
	t_color *col;
	VALUE str;

	s = ALLOCA_N(char, 256);
	Data_Get_Struct(obj, t_color, col);
	sprintf(s, "#<Xlib::Color pixel=%ld red=0x%x green=0x%x blue=0x%x>", col->pixel, col->red, col->green, col->blue);
	str = rb_str_new2(s);

	return str;
}

void
xlib_init_color()
{
	cXlib_color = rb_define_class_under(mXlib, "Color", rb_cObject);
	rb_define_singleton_method(cXlib_color, "new", xlib_color_new, -1);
	SetupAttr(color, pixel)
	SetupAttr(color, red)
	SetupAttr(color, green)
	SetupAttr(color, blue)
	rb_define_method(cXlib_color, "to_s", xlib_color_to_s, 0);
}
