/**
 * cursor.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"

#include "display.h"
#include "window.h"
#include "font.h"
#include "color.h"
#include "point.h"
#include "cursor.h"

void
xlib_free_cursor(cursor)
	t_cursor *cursor;
{
	free(cursor);
}

static VALUE
xlib_cursor_new_from_shape(argc, argv, self)
	int    argc;
	VALUE *argv;
	VALUE  self;
{
	VALUE obj;
	VALUE display, shape;
	t_display *disp;
	t_cursor *curp;
	Cursor xcursor;
	int cnt;

	cnt = rb_scan_args(argc, argv, "20", &display, &shape);
	GetDisplay(display, disp);
	xcursor = XCreateFontCursor(disp->xdisplay, NUM2INT(shape));
	obj = Data_Make_Struct(self, t_cursor, 0, xlib_free_cursor, curp);
	curp->xcursor = xcursor;

	return obj;
}

static VALUE
xlib_cursor_new_from_glyph(argc, argv, self)
	int argc;
	VALUE *argv;
	VALUE self;
{
	VALUE obj;
	VALUE display, face_font, mask_font, face, mask, fg, bg;
	t_display *disp;
	t_cursor *curp;
	Cursor xcursor;
	t_font *fonp;
	XFontStruct **data;
	Font face_fid, mask_fid;
	char **name;
	t_color *_fg, *_bg;
	int cnt;

	cnt = rb_scan_args(argc, argv, "70", &display,
	  &face_font, &mask_font,
	  &face, &mask,
	  &fg, &bg);
	GetDisplay(display, disp);

	GetFont(face_font, fonp);
	if (fonp->nfont < 1) {
		rb_raise(rb_eRuntimeError, "nfont less than 1");
	}
	XFontsOfFontSet(fonp->xfont, &data, &name);
	face_fid = data[0]->fid;

	GetFont(mask_font, fonp);
	if (fonp->nfont < 1) {
		rb_raise(rb_eRuntimeError, "nfont less than 1");
	}
	XFontsOfFontSet(fonp->xfont, &data, &name);
	mask_fid = data[0]->fid;

	Data_Get_Struct(fg, t_color, _fg);
	Data_Get_Struct(bg, t_color, _bg);
	xcursor = XCreateGlyphCursor(disp->xdisplay,
	  face_fid, mask_fid,
	  NUM2INT(face), NUM2INT(mask),
	  _fg, _bg);
	obj = Data_Make_Struct(self, t_cursor, 0, xlib_free_cursor, curp);
	curp->xcursor = xcursor;

	return obj;
}

static VALUE
xlib_cursor_new(argc, argv, self)
	int    argc;
	VALUE *argv;
	VALUE  self;
{
	VALUE obj;
	t_display *disp;
	t_cursor *curp;
	Cursor xcursor;
	int cnt;
	VALUE display, pmap, mask, fg, bg, xhot, yhot;
	t_window *_pmap, *_mask;
	t_color *_fg, *_bg;
	unsigned int _xhot, _yhot;

	cnt = rb_scan_args(argc, argv, "52", &display,
	  &pmap, &mask,
	  &fg, &bg,
	  &xhot, &yhot);
	GetDisplay(display, disp);
	GetWindow(pmap, _pmap);
	GetWindow(mask, _mask);
	Data_Get_Struct(fg, t_color, _fg);
	Data_Get_Struct(bg, t_color, _bg);
	if (cnt == 7) {
		_xhot = NUM2INT(xhot);
		_yhot = NUM2INT(yhot);
	} else {
		VALUE hotspot;
		t_point *hotp;

		hotspot = rb_iv_get(pmap, "@hotspot");
		Data_Get_Struct(hotspot, t_point, hotp);
		_xhot = hotp->x;
		_yhot = hotp->y;
	}
	xcursor = XCreatePixmapCursor(disp->xdisplay,
	  _pmap->xwindow, _mask->xwindow,
	  _fg, _bg,
	  _xhot, _yhot);
	obj = Data_Make_Struct(self, t_cursor, 0, xlib_free_cursor, curp);
	curp->xcursor = xcursor;

	return obj;
}

void
xlib_init_cursor()
{
	cXlib_cursor = rb_define_class_under(mXlib, "Cursor", rb_cObject);
	rb_define_singleton_method(cXlib_cursor, "new_from_shape", xlib_cursor_new_from_shape, -1);
	rb_define_singleton_method(cXlib_cursor, "new_from_glyph", xlib_cursor_new_from_glyph, -1);
	rb_define_singleton_method(cXlib_cursor, "new", xlib_cursor_new, -1);
}
