/**
 * border.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"
#include "attr.h"
#include "border.h"

static void
im_free_border(border)
	t_border *border;
{
	free(border);
}

static VALUE
im_border_new(argc, argv, class)
   int argc;
   VALUE *argv;
   VALUE class;
{
   t_border *border;
   VALUE obj, left, top, right, bottom;
   
   obj = Data_Make_Struct(class, t_border, 0, im_free_border, border);
   if (rb_scan_args(argc, argv, "04", &left, &top, &right, &bottom) == 4) {
      border->left = NUM2INT(left);
      border->top = NUM2INT(top);
      border->right = NUM2INT(right);
      border->bottom = NUM2INT(bottom);
   }
   
   rb_obj_call_init(obj, argc, argv);
   
   return obj;
}

DefineNumericAttr(border, left)
DefineNumericAttr(border, top)
DefineNumericAttr(border, right)
DefineNumericAttr(border, bottom)

static VALUE
im_border_to_s(obj)
   VALUE obj;
{
   char *s;
   t_border *border;
   VALUE str;

   s = ALLOCA_N(char, 20);
   Data_Get_Struct(obj, t_border, border);
   sprintf(s, "<#Imlib::Border left=%d top=%d right=%d bottom=%d>",
      border->left,
      border->top,
      border->right,
      border->bottom );
   str = rb_str_new2(s);
   return str;
}

void
im_init_border()
{
   cImlib_border = rb_define_class_under(mImlib, "Border", rb_cObject);
   rb_define_singleton_method(cImlib_border, "new", im_border_new, -1);
   SetupAttr(border, left)
   SetupAttr(border, top)
   SetupAttr(border, right)
   SetupAttr(border, bottom)
   rb_define_method(cImlib_border, "to_s", im_border_to_s, 0);
}
