/**
 * color.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"
#include "attr.h"
#include "color.h"

static void
im_free_color(col)
	t_color *col;
{
	free(col);
}

static VALUE
im_color_new(argc, argv, class)
   int argc;
   VALUE *argv;
   VALUE class;
{
   t_color *color;
   VALUE obj, r, g, b, pixel;
   
   obj = Data_Make_Struct(class, t_color, 0, im_free_color, color);
   if (rb_scan_args(argc, argv, "04", &r, &g, &b, &pixel) == 4) {
      color->r = NUM2INT(r);
      color->g = NUM2INT(g);
      color->b = NUM2INT(b);
      color->pixel = NUM2INT(pixel);
   }
   
   rb_obj_call_init(obj, argc, argv);
   
   return obj;
}

DefineNumericAttr(color, r)
DefineNumericAttr(color, g)
DefineNumericAttr(color, b)
DefineNumericAttr(color, pixel)

static VALUE
im_color_to_s(obj)
   VALUE obj;
{
   char *s;
   t_color *color;
   VALUE str;

   s = ALLOCA_N(char, 20);
   Data_Get_Struct(obj, t_color, color);
   sprintf(s, "<#Imlib::Border r=%d b=%d g=%d pixel=%d>",
      color->r,
      color->b,
      color->g,
      color->pixel );
   str = rb_str_new2(s);
   return str;
}

void
im_init_color()
{
   cImlib_color = rb_define_class_under(mImlib, "Color", rb_cObject);
   rb_define_singleton_method(cImlib_color, "new", im_color_new, -1);
   SetupAttr(color, r)
   SetupAttr(color, g)
   SetupAttr(color, b)
   SetupAttr(color, pixel)
   rb_define_method(cImlib_color, "to_s", im_color_to_s, 0);
}
