/**
 * image.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"
#include "data.h"
#include "image.h"
#include "border.h"
#include "color.h"
#include "modifier.h"
#include "window.h"
#include "pixmap.h"
#include "rect.h"
#include "debug.h"

void
im_no_image()
{
    rb_raise(rb_eRuntimeError, "already closed image");
}

void
im_free_image(imp)
	t_image *imp;
{
	Debug("im free image begin");
	if (imp->idp && imp->idp->id && imp->im) {
		Imlib_destroy_image(imp->idp->id, imp->im);
		imp->im = 0;
	}
	/* free(imp); */
	Debug("im free image end");
}

VALUE
im_image_new(class, data)
	VALUE class, data;
{
	t_data *idp;
	t_image *imp;
	VALUE obj;
	
	GetData(data, idp);
	obj = Data_Make_Struct(class, t_image, 0, im_free_image, imp);
	imp->prev = idp->im_list_head;
	imp->next = idp->im_list_head->next;
	idp->im_list_head->next->prev = imp;
	idp->im_list_head->next = imp;
	imp->idp = idp;
	rb_iv_set(obj, "@data", data);
	
	return obj;
}

static VALUE
im_image_render(obj, width, height)
	VALUE obj, width, height;
{
	t_image *imp;
	int ret;
	
	GetImage(obj, imp);
	ret = Imlib_render(imp->idp->id, imp->im, NUM2INT(width), NUM2INT(height));
	return ret ? Qtrue : Qfalse;
}

static VALUE
im_image_copy(obj)
	VALUE obj;
{
	t_image *imp;
	Pixmap pmap, mask;
	VALUE  pmap_obj, mask_obj, display, data;
	
	GetImage(obj, imp);
	pmap = Imlib_copy_image(imp->idp->id, imp->im);
	mask = Imlib_copy_mask (imp->idp->id, imp->im);
	
	data = rb_iv_get(obj, "@data");
	display = rb_iv_get(data, "@display");
	
	pmap_obj = pmap ? im_pixmap_create(pmap, display, imp->idp) : Qnil;
	mask_obj = mask ? im_pixmap_create(mask, display, imp->idp) : Qnil;
	
	return rb_assoc_new(pmap_obj, mask_obj);
}

static VALUE
im_image_move(obj)
	VALUE obj;
{
	t_image *imp;
	Pixmap pmap, mask;
	VALUE pmap_obj, mask_obj;
	VALUE data, display;
	
	GetImage(obj, imp);
	pmap = Imlib_move_image(imp->idp->id, imp->im);
	mask = Imlib_move_mask (imp->idp->id, imp->im);
	
	data = rb_iv_get(obj, "@data");
	display = rb_iv_get(data, "@display");
	
	pmap_obj = pmap ? im_pixmap_create(pmap, display, imp->idp): Qnil;
	mask_obj = mask ? im_pixmap_create(mask, display, imp->idp): Qnil;
	
	return rb_assoc_new(pmap_obj, mask_obj);
}

static VALUE
im_image_changed(obj)
	VALUE obj;
{
	t_image *imp;
	GetImage(obj, imp);
	Imlib_changed_image(imp->idp->id, imp->im);
	return Qnil;
}

static VALUE
im_image_destroy(obj)
	VALUE obj;
{
	t_image *imp;
	
	GetImage(obj, imp);
	Imlib_destroy_image(imp->idp->id, imp->im);
	imp->im = 0;
	
	return Qnil;
}

static VALUE
im_image_kill(obj)
	VALUE obj;
{
	t_image *imp;
	
	GetImage(obj, imp);
	Imlib_kill_image(imp->idp->id, imp->im);
	imp->im = 0;
	
	return Qnil;
}

static VALUE
im_image_get_border(obj)
	VALUE obj;
{
	t_border *border = NULL;
	VALUE border_obj;
	t_image *imp;
	
	GetImage(obj, imp);
	
	Imlib_get_image_border(imp->idp->id, imp->im, border);
	border_obj = Data_Wrap_Struct(cImlib_border, 0, 0, border);
	
	return border_obj;
}

static VALUE
im_image_set_border(obj, border_obj)
	VALUE obj, border_obj;
{
	t_border *border;
	t_image *imp;
	
	GetImage(obj, imp);
	
	Data_Get_Struct(border_obj, t_border, border);
	Imlib_set_image_border(imp->idp->id, imp->im, border);
	
	return Qnil;
}

static VALUE
im_image_get_shape(obj)
	VALUE obj;
{
	t_color *col = NULL;
	VALUE col_obj;
	t_image *imp;
	
	GetImage(obj, imp);
	
	Imlib_get_image_shape(imp->idp->id, imp->im, col);
	col_obj = Data_Wrap_Struct(cImlib_color, 0, 0, col);
	
	return col_obj;
}

static VALUE
im_image_set_shape(obj, col_obj)
	VALUE obj, col_obj;
{
	t_color *col;
	t_image *imp;
	
	GetImage(obj, imp);
	
	Data_Get_Struct(col_obj, t_color, col);
	Imlib_set_image_shape(imp->idp->id, imp->im, col);
	
	return Qnil;
}

static VALUE
im_image_save_to_eim(obj, file)
	VALUE obj, file;
{
	t_image *imp;
	int ret;
	
	GetImage(obj, imp);
	ret = Imlib_save_image_to_eim(imp->idp->id, imp->im, STR2CSTR(file));
	return ret ? Qtrue : Qfalse;
	return Qnil;
}

static VALUE
im_image_add_to_eim(obj, file)
	VALUE obj, file;
{
	t_image *imp;
	int ret;
	
	GetImage(obj, imp);
	ret = Imlib_add_image_to_eim(imp->idp->id, imp->im, STR2CSTR(file));
	return ret ? Qtrue : Qfalse;
	return Qnil;
}

static VALUE
im_image_save_to_ppm(obj, file)
	VALUE obj, file;
{
	t_image *imp;
	int ret;
	
	GetImage(obj, imp);
	ret = Imlib_save_image_to_ppm(imp->idp->id, imp->im, STR2CSTR(file));
	return ret ? Qtrue : Qfalse;
	return Qnil;
}

static VALUE
im_image_get_modifier(obj)
	VALUE obj;
{
	t_modifier *mod = NULL;
	VALUE mod_obj;
	t_image *imp;
	
	GetImage(obj, imp);
	
	Imlib_get_image_modifier(imp->idp->id, imp->im, mod);
	mod_obj = Data_Wrap_Struct(cImlib_modifier, 0, im_free_modifier, mod);
	
	return mod_obj;
}

static VALUE
im_image_set_modifier(obj, modifier_obj)
	VALUE obj, modifier_obj;
{
	t_modifier *modifier;
	t_image *imp;
	
	GetImage(obj, imp);
	
	Data_Get_Struct(modifier_obj, t_modifier, modifier);
	Imlib_set_image_modifier(imp->idp->id, imp->im, modifier);
	
	return Qnil;
}

#define DefineCurveAttr(axis) \
static VALUE \
im_image_get_##axis##_curve(obj)\
	VALUE obj;\
{\
	VALUE str;\
	t_image *imp;\
	char *s = "";\
	\
	GetImage(obj, imp);\
	\
	Imlib_get_image_##axis##_curve(imp->idp->id, imp->im, s);\
	str = rb_str_new2(s);\
	\
	return str;\
}\
\
static VALUE \
im_image_set_##axis##_curve(obj, str)\
	VALUE obj, str;\
{\
	t_image *imp;\
	char *s;\
	\
	GetImage(obj, imp);\
	s = STR2CSTR(str);\
	\
	Imlib_set_image_##axis##_curve(imp->idp->id, imp->im, s);\
	\
	return Qnil;\
}

DefineCurveAttr(red)
DefineCurveAttr(green)
DefineCurveAttr(blue)

static VALUE
im_image_apply_modifiers_to_rgb(obj)
	VALUE obj;
{
	t_image *imp;
	GetImage(obj, imp);
	Imlib_apply_modifiers_to_rgb(imp->idp->id, imp->im);
	return Qnil;
}

static VALUE
im_image_apply(obj, win)
	VALUE obj, win;
{
	t_image *imp;
	t_window *winp;

	GetImage(obj, imp);
	GetWindow(win, winp);
	Imlib_apply_image(imp->idp->id, imp->im, winp->xwindow);

	return Qnil;
}

static VALUE
im_image_paste(obj, win, rect_obj, is_border)
	VALUE obj, win, rect_obj, is_border;
{
	t_image *imp;
	t_window *winp;
	t_rect *rect;
	
	GetImage(obj, imp);
	GetWindow(win, winp);
	Data_Get_Struct(rect_obj, t_rect, rect);
	
	if (is_border)
		Imlib_paste_image_border(imp->idp->id, imp->im, winp->xwindow, rect->x, rect->y, rect->width, rect->height);
	else
		Imlib_paste_image(imp->idp->id, imp->im, winp->xwindow, rect->x, rect->y, rect->width, rect->height);
	return Qnil;
}

static VALUE
im_image_flip(obj, axis)
	VALUE obj, axis;
{
	t_image *imp;
	
	GetImage(obj, imp);
	
	switch (NUM2INT(axis)) {
	case 'h':
	case 'H':
		Imlib_flip_image_horizontal(imp->idp->id, imp->im);
		break;
	case 'v':
	case 'V':
		Imlib_flip_image_vertical(imp->idp->id, imp->im);
		break;
	}
	
	return Qnil;
}

static VALUE
im_image_rotate(obj, d)
	VALUE obj, d;
{
	t_image *imp;
	GetImage(obj, imp);
	Imlib_rotate_image(imp->idp->id, imp->im, NUM2INT(d));
	return Qnil;
}

static VALUE
im_image_clone(argc, argv, image)
	int argc;
	VALUE *argv;
	VALUE image;
{
	t_image *imp;
	t_image *clp;
	VALUE data, clone, width, height;
	int cnt;
	
	GetImage(image, imp);
	cnt = rb_scan_args(argc, argv, "02", &width, &height);
	data = rb_iv_get(image, "@data");
	clone = im_image_new(cImlib_image, data);
	Data_Get_Struct(clone, t_image, clp);
	switch (cnt) {
	case 0:
		clp->im = Imlib_clone_image(imp->idp->id, imp->im);
		break;
	case 2:
		clp->im = Imlib_clone_scaled_image(imp->idp->id, imp->im, NUM2INT(width), NUM2INT(height));
		break;
	default:
		rb_raise(rb_eArgError, "argc must be 0 or 2");
	}
	
	return clone;
}

static VALUE
im_image_crop(obj, area)
	VALUE obj, area;
{
	t_image *imp;
	t_rect *rect;
	GetImage(obj, imp);
	Data_Get_Struct(area, t_rect, rect);
	Imlib_crop_image(imp->idp->id, imp->im,
	  rect->x, rect->y, rect->width, rect->height);
	return Qnil;
}

static VALUE
im_image_crop_and_clone(image, area)
	VALUE image, area;
{
	t_image *imp;
	t_image *clp;
	t_rect *rect;
	VALUE data, clone;
	
	GetImage(image, imp);
	Data_Get_Struct(area, t_rect, rect);
	data = rb_iv_get(image, "@data");
	clone = im_image_new(cImlib_image, data);
	Data_Get_Struct(clone, t_image, clp);
	clp->im = Imlib_crop_and_clone_image(imp->idp->id, imp->im,
	  rect->x, rect->y, rect->width, rect->height);
	
	return clone;
}

static VALUE
im_image_save(obj, file)
	VALUE obj, file;
{
	t_image *imp;
	int ret;
	GetImage(obj, imp);
	ret = Imlib_save_image(imp->idp->id, imp->im,
	  STR2CSTR(file),
	  NULL /*  default info */);
	return ret ? Qtrue : Qfalse;
}

/*
  ImlibImage         *Imlib_inlined_png_to_image(ImlibData *id, unsigned char *data, int data_size);
*/

static VALUE
im_image_get_width(obj)
	VALUE obj;
{
	t_image *imp;
	GetImage(obj, imp);
	return INT2NUM(imp->im->rgb_width);
}

static VALUE
im_image_get_height(obj)
	VALUE obj;
{
	t_image *imp;
	GetImage(obj, imp);
	return INT2NUM(imp->im->rgb_height);
}

void
im_init_image()
{
	cImlib_image = rb_define_class_under(mImlib, "Image", rb_cObject);
	DefineMethod(image, render, 2)
	DefineMethod(image, copy, 0)
	DefineMethod(image, move, 0)
	DefineMethod(image, changed, 0)
	DefineMethod(image, destroy, 0)
	DefineMethod(image, kill, 0)
	DefineMethodGet(image, border)
	DefineMethodSet(image, border)
	DefineMethodGet(image, shape)
	DefineMethodSet(image, shape)
	DefineMethod(image, save_to_eim, 1)
	DefineMethod(image, add_to_eim, 1)
	DefineMethod(image, save_to_ppm, 1)
	DefineMethodGet(image, modifier)
	DefineMethodSet(image, modifier)
	DefineMethodGet(image, red_curve)
	DefineMethodSet(image, red_curve)
	DefineMethodGet(image, green_curve)
	DefineMethodSet(image, green_curve)
	DefineMethodGet(image, blue_curve)
	DefineMethodSet(image, blue_curve)
	DefineMethod(image, apply_modifiers_to_rgb, 0)
	DefineMethod(image, apply, 1)
	DefineMethod(image, paste, 3)
	DefineMethod(image, flip, 1)
	DefineMethod(image, rotate, 1)
	DefineMethod(image, clone, -1)
	DefineMethod(image, crop, 1)
	DefineMethod(image, crop_and_clone, 1)
	DefineMethod(image, save, 1)
	DefineMethodGet(image, width)
	DefineMethodGet(image, height)
	rb_define_attr(cImlib_image, "data", 1, 0);
}
