/**
 * modifier.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"
#include "attr.h"
#include "color.h"
#include "modifier.h"

void
im_free_modifier(mod)
	t_modifier *mod;
{
	free(mod);
}

static VALUE
im_modifier_new(argc, argv, class)
   int argc;
   VALUE *argv;
   VALUE class;
{
   t_modifier *modifier;
   VALUE obj, gamma, brightness, contrast;
   
   obj = Data_Make_Struct(class, t_modifier, 0, im_free_modifier, modifier);
   if (rb_scan_args(argc, argv, "03", &gamma, &brightness, &contrast) == 4) {
      modifier->gamma = NUM2INT(gamma);
      modifier->brightness = NUM2INT(brightness);
      modifier->contrast = NUM2INT(contrast);
   }
   
   rb_obj_call_init(obj, argc, argv);
   
   return obj;
}

DefineNumericAttr(modifier, gamma)
DefineNumericAttr(modifier, brightness)
DefineNumericAttr(modifier, contrast)

static VALUE
im_modifier_to_s(obj)
   VALUE obj;
{
   char *s;
   t_modifier *modifier;
   VALUE str;

   s = ALLOCA_N(char, 20);
   Data_Get_Struct(obj, t_modifier, modifier);
   sprintf(s, "<#Imlib::Border gamma=%d brightness=%d contrast=%d>",
      modifier->gamma,
      modifier->brightness,
      modifier->contrast );
   str = rb_str_new2(s);
   return str;
}

void
im_init_modifier()
{
   cImlib_modifier = rb_define_class_under(cImlib_color, "Modifier", rb_cObject);
   rb_define_singleton_method(cImlib_modifier, "new", im_modifier_new, -1);
   SetupAttr(modifier, gamma)
   SetupAttr(modifier, brightness)
   SetupAttr(modifier, contrast)
   rb_define_method(cImlib_modifier, "to_s", im_modifier_to_s, 0);
}
