/*
 * pixmap.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"
#include "data.h"
#include "display.h"
#include "window.h"
#include "debug.h"

void
im_free_pixmap(pixp)
	t_window *pixp;
{
	Debug("im free pixmap begin");
	if (pixp->xwindow) {
		Imlib_free_pixmap(pixp->idp->id, pixp->xwindow);
		pixp->xwindow = 0;
	}
	pixp->next->prev = pixp->prev;
	pixp->prev->next = pixp->next;
	free(pixp);
	Debug("im free pixmap end");
}

VALUE
im_pixmap_close(obj)
	VALUE obj;
{
	t_window *pixp;
	
	GetWindow(obj, pixp);
	Imlib_free_pixmap(pixp->idp->id, pixp->xwindow);
	pixp->xwindow = 0;
	
	return Qnil;
}

VALUE
im_pixmap_create(xpixmap, display, idp)
	Pixmap xpixmap;
	VALUE  display;
	t_data *idp;
{
	t_display *disp;
	t_window  *pixp;
	VALUE obj;
	
	GetDisplay(display, disp);
	obj = Data_Make_Struct(cXlib_pixmap, t_window, 0, im_free_pixmap, pixp);
	pixp->prev = idp->pixmap_list_head;
	pixp->next = idp->pixmap_list_head->next;
	idp->pixmap_list_head->next->prev = pixp;
	idp->pixmap_list_head->next = pixp;
	pixp->xdisplay = disp->xdisplay;
	pixp->xparent  = 0;
	pixp->xwindow  = xpixmap;
	pixp->ic = 0;
	pixp->idp = idp;
	rb_iv_set(obj, "@display", display);
	rb_define_singleton_method(obj, "close", im_pixmap_close, 0);
	
	return obj;
}
