/**
 * init.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"

#include <locale.h>
#include <X11/keysym.h>

#include "display.h"
#include "window.h"
#include "font.h"
#include "gc.h"
#include "color.h"
#include "point.h"
#include "rect.h"
#include "im.h"
#include "cursor.h"
#include "icon.h"
#include "image.h"
#include "event.h"
#include "pixmap_format.h"
#include "time_coord.h"

#ifdef USE_RUBY_GTK
#include "global.h"
#endif

static VALUE
xlib_cast(self, orig, xlib)
	VALUE self, orig, xlib;
{
	VALUE obj;

	obj = Qnil;

#ifdef USE_RUBY_GTK
	if (rb_obj_is_kind_of(orig, gdkPixmap)) {
		GdkWindowPrivate *private;

		private = (GdkWindowPrivate *)(get_gdkpixmap(orig));
		obj = xlib_pixmap_setup(xlib, private->xwindow, -1, -1);
	}
#endif
	return obj;
}

void
Init_xlib()
{
	setlocale(LC_ALL, "");
	XSetLocaleModifiers("");

	id_init = rb_intern("initialize");
	id_call = rb_intern("call");

	mXlib = rb_define_module("Xlib");
	rb_define_module_function(mXlib, "cast", xlib_cast, 2);
	xlib_init_display();
	xlib_init_window();
	xlib_init_pixmap();
	xlib_init_gc();
	xlib_init_font();
	xlib_init_color();
	xlib_init_point();
	xlib_init_rect();
	xlib_init_im();
	xlib_init_visual();
	xlib_init_cursor();
	xlib_init_icon();
	xlib_init_image();
	xlib_init_event();
	xlib_init_pixmap_format();
	xlib_init_time_coord();

	rb_define_const(mXlib, "VERSION_MEJOR", INT2FIX(0));
	rb_define_const(mXlib, "VERSION_MINOR", INT2FIX(6));
	rb_define_const(mXlib, "REVISION", INT2FIX(13));

	/* graphics functions */
	rb_define_const(mXlib, "CLEAR", INT2FIX(GXclear));
	rb_define_const(mXlib, "AND", INT2FIX(GXand));
	rb_define_const(mXlib, "AND_REVERSE", INT2FIX(GXandReverse));
	rb_define_const(mXlib, "COPY", INT2FIX(GXcopy));
	rb_define_const(mXlib, "AND_INVERT", INT2FIX(GXandInverted));
	rb_define_const(mXlib, "NOOP", INT2FIX(GXnoop));
	rb_define_const(mXlib, "XOR", INT2FIX(GXxor));
	rb_define_const(mXlib, "OR", INT2FIX(GXor));
	rb_define_const(mXlib, "NOR", INT2FIX(GXnor));
	rb_define_const(mXlib, "EQUIV", INT2FIX(GXequiv));
	rb_define_const(mXlib, "OR_REVERSE", INT2FIX(GXorReverse));
	rb_define_const(mXlib, "COPY_INVERT", INT2FIX(GXcopyInverted));
	rb_define_const(mXlib, "OR_INVERT", INT2FIX(GXorInverted));
	rb_define_const(mXlib, "NAND", INT2FIX(GXnand));
	rb_define_const(mXlib, "SET", INT2FIX(GXset));

	/* line_style */
	rb_define_const(mXlib, "LINE_SOLID", INT2FIX(LineSolid));
	rb_define_const(mXlib, "LINE_ON_OFF_DASH", INT2FIX(LineOnOffDash));
	rb_define_const(mXlib, "LINE_DOUBLE_DASH", INT2FIX(LineDoubleDash));

	/* cap_style */
	rb_define_const(mXlib, "CAP_NOTLAST", INT2FIX(CapNotLast));
	rb_define_const(mXlib, "CAP_BUTT", INT2FIX(CapButt));
	rb_define_const(mXlib, "CAP_ROUND", INT2FIX(CapRound));
	rb_define_const(mXlib, "CAP_PROJECT", INT2FIX(CapProjecting));

	/* join_style */
	rb_define_const(mXlib, "JOIN_MITER", INT2FIX(JoinMiter));
	rb_define_const(mXlib, "JOIN_ROUND", INT2FIX(JoinRound));
	rb_define_const(mXlib, "JOIN_BEVEL", INT2FIX(JoinBevel));

	/* coordinate_mode for drawing routines */
	rb_define_const(mXlib, "COORD_ORIG", INT2FIX(CoordModeOrigin));
	rb_define_const(mXlib, "COORD_PREV", INT2FIX(CoordModePrevious));

	/* polygon shapes */
	rb_define_const(mXlib, "POLY_FREE", INT2FIX(Complex));
	rb_define_const(mXlib, "POLY_NONE", INT2FIX(Nonconvex));
	rb_define_const(mXlib, "POLY_CONVEX", INT2FIX(Convex));
}
