/**
 * init.h
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#define DefIntern(str) static ID id_##str;
#define SetIntern(str) id_##str = rb_intern(#str);
#define DefineSingletonMethod(class, method, argc) rb_define_singleton_method(cXlib_##class, #method, xlib_##class##_##method, argc);
#define DefineMethod(class, method, argc) rb_define_method(cXlib_##class, #method, xlib_##class##_##method, argc);
#define DefineMethodGet(class, method) rb_define_method(cXlib_##class, #method, xlib_##class##_get_##method, 0);
#define DefineMethodIs(class, method) rb_define_method(cXlib_##class, #method "?", xlib_##class##_is_##method, 0);
#define DefineMethodSet(class, method) rb_define_method(cXlib_##class, #method "=", xlib_##class##_set_##method, 1);

VALUE mXlib;
ID id_init;
ID id_call;

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#ifdef HAVE_IMLIB_H
#include <Imlib.h>
#include "imlib/data.h"
#include "imlib/image.h"
#endif

#ifdef USE_RUBY_GTK
VALUE gdkPixmap;
#endif

#define xlib_raise_if_argc_rest(argc)\
{\
	if ( argc ) rb_raise(rb_eArgError, "too many arguments");\
}

#define xlib_raise_if_argc_none(argc, info)\
{\
	if (!argc ) rb_raise(rb_eArgError, "wrong argument (" info ")");\
}

#define xlib_scan_args_num(argc, argv, _n)\
{\
	xlib_raise_if_argc_none(argc, "1 Fixnum");\
	_n = NUM2INT(*argv);\
	if (--argc) ++argv;\
}

#define xlib_scan_args_num_nil(argc, argv, _n, _nil)\
{\
	if (argc && !NIL_P(*argv)) {\
		_n = NUM2INT(*argv);\
	} else {\
		_n = _nil;\
	}\
	if (argc && --argc) ++argv;\
}

#define xlib_scan_args_point(argc, argv, _x, _y)\
{\
	xlib_raise_if_argc_none(argc, "need 1 Point or 2 Fixnum");\
	if (rb_obj_is_kind_of(*argv, cXlib_point)) {\
		t_point *point;\
\
		Data_Get_Struct(*argv, t_point, point);\
		if (--argc) ++argv;\
		_x = point->x;\
		_y = point->y;\
	} else {\
		_x = NUM2INT(*argv);\
		if (--argc) ++argv;\
		xlib_raise_if_argc_none(argc, "1 Fixnum for 2 Fixnum");\
		_y = NUM2INT(*argv);\
		if (--argc) ++argv;\
	}\
}

#define xlib_scan_args_point_nil(argc, argv, _x, _y)\
{\
	if (!argc) {\
		_x = 0;\
		_y = 0;\
	} else if (rb_obj_is_kind_of(*argv, cXlib_point)) {\
		t_point *point;\
\
		Data_Get_Struct(*argv, t_point, point);\
		if (--argc) ++argv;\
		_x = point->x;\
		_y = point->y;\
	} else {\
		_x = NUM2INT(*argv);\
		if (--argc) ++argv;\
		xlib_raise_if_argc_none(argc, "1 Fixnum for 2 Fixnum");\
		_y = NUM2INT(*argv);\
		if (--argc) ++argv;\
	}\
}

#define xlib_scan_args_rect(argc, argv, _x, _y, _width, _height)\
{\
	xlib_raise_if_argc_none(argc, "need 1 Rect or 2 Fixnum");\
	if (rb_obj_is_kind_of(*argv, cXlib_rect)) {\
		t_rect *rect;\
\
		Data_Get_Struct(*argv, t_rect, rect);\
		if (--argc) ++argv;\
		_x = rect->x;\
		_y = rect->y;\
		_width  = rect->width ;\
		_height = rect->height;\
	} else {\
		_x = NUM2INT(*argv);\
		if (--argc) ++argv;\
		xlib_raise_if_argc_none(argc, "1 Fixnum for 4 Fixnum");\
		_y = NUM2INT(*argv);\
		if (--argc) ++argv;\
		xlib_raise_if_argc_none(argc, "2 Fixnum for 4 Fixnum");\
		_width  = NUM2INT(*argv);\
		if (--argc) ++argv;\
		xlib_raise_if_argc_none(argc, "3 Fixnum for 4 Fixnum");\
		_height = NUM2INT(*argv);\
		if (--argc) ++argv;\
	}\
}

#define xlib_scan_args_rect_size(argc, argv, _x, _y, _width, _height)\
{\
	xlib_raise_if_argc_none(argc, "need 1 Rect or 2 Fixnum");\
	if (rb_obj_is_kind_of(*argv, cXlib_rect)) {\
		t_rect *rect;\
\
		Data_Get_Struct(*argv, t_rect, rect);\
		if (--argc) ++argv;\
		_x = rect->x;\
		_y = rect->y;\
		_width  = rect->width ;\
		_height = rect->height;\
	} else {\
		_x = 0;\
		_y = 0;\
		_width  = NUM2INT(*argv);\
		if (--argc) ++argv;\
		xlib_raise_if_argc_none(argc, "1 Fixnum for 2 Fixnum");\
		_height = NUM2INT(*argv);\
		if (--argc) ++argv;\
	}\
}

void Init_xlib();

