/**
 * point.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"

#include "attr.h"
#include "point.h"

static void
xlib_free_point(point)
	t_point *point;
{
	XFree(point);
}

static VALUE
xlib_point_new(argc, argv, class)
	int argc;
	VALUE *argv;
	VALUE class;
{
	t_point *point;
	VALUE obj, x, y;
	int cnt;
	
	obj = Data_Make_Struct(class, t_point, 0, xlib_free_point, point);
	cnt = rb_scan_args(argc, argv, "02", &x, &y);
	switch (cnt) {
	case 2:
		point->x = NUM2INT(x);
		point->y = NUM2INT(y);
		break;
	default:
		point->x = 0;
		point->y = 0;
		break;
	}
	
	rb_obj_call_init(obj, 0, NULL);
	
	return obj;
}

DefineNumericAttr(point, x)
DefineNumericAttr(point, y)

static VALUE
xlib_point_to_s(obj)
	VALUE obj;
{
	char *s;
	t_point *point;
	VALUE str;

	s = ALLOCA_N(char, 256);
	Data_Get_Struct(obj, t_point, point);
	sprintf(s, "#<Xlib::Point x=%d y=%d>", point->x, point->y);
	str = rb_str_new2(s);

	return str;
}

void
xlib_init_point()
{
	cXlib_point = rb_define_class_under(mXlib, "Point", rb_cObject);
	rb_define_singleton_method(cXlib_point, "new", xlib_point_new, -1);
	SetupAttr(point, x)
	SetupAttr(point, y)
	rb_define_method(cXlib_point, "to_s", xlib_point_to_s, 0);
}
