/**
 * rect.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"

#include "attr.h"
#include "rect.h"

void
xlib_free_rect(rect)
	t_rect *rect;
{
	XFree(rect);
}

static VALUE
xlib_rect_new(argc, argv, class)
	int argc;
	VALUE *argv;
	VALUE class;
{
	t_rect *rect;
	VALUE obj, x, y, width, height;
	int cnt;
	
	obj = Data_Make_Struct(class, t_rect, 0, xlib_free_rect, rect);
	cnt = rb_scan_args(argc, argv, "04", &x, &y, &width, &height); 
	switch (cnt) {
	case 4:
		rect->x = NUM2INT(x);
		rect->y = NUM2INT(y);
		rect->width = NUM2INT(width );
		rect->height = NUM2INT(height);
		break;
	default:
		rect->x = 0;
		rect->y = 0;
		rect->width = 0;
		rect->height = 0;
		break;
	}
	
	rb_obj_call_init(obj, 0, NULL);
	
	return obj;
}

DefineNumericAttr(rect, x)
DefineNumericAttr(rect, y)
DefineNumericAttr(rect, width )
DefineNumericAttr(rect, height)

static VALUE
xlib_rect_to_s(obj)
	VALUE obj;
{
	char *s;
	t_rect *rect;
	VALUE str;

	s = ALLOCA_N(char, 256);
	Data_Get_Struct(obj, t_rect, rect);
	sprintf(s, "#<Xlib::Rect x=%d y=%d width=%d height=%d>", rect->x, rect->y, rect->width, rect->height);
	str = rb_str_new2(s);

	return str;
}

static VALUE
xlib_rect_clone(obj)
	VALUE obj;
{
	t_rect *a, *c;
	VALUE clone;

	Data_Get_Struct(obj, t_rect, a);
	clone = Data_Make_Struct(CLASS_OF(obj), t_rect, 0, xlib_free_rect, c);

	c->x = a->x;
	c->y = a->y;
	c->width = a->width ;
	c->height = a->height;

	return clone;
}

void
xlib_init_rect()
{
	cXlib_rect = rb_define_class_under(mXlib, "Rect", rb_cObject);
	rb_define_singleton_method(cXlib_rect, "new", xlib_rect_new, -1);
	SetupAttr(rect, x)
	SetupAttr(rect, y)
	SetupAttr(rect, width )
	SetupAttr(rect, height)
	DefineMethod(rect, to_s, 0)
	DefineMethod(rect, clone, 0)
}
