require 'xlib'
include  Xlib

$display = Display.new

module Util
  def abs(x) if x<0 then -x else x end end
  def less(a, b) if a<b then a else b end end
  def more(a, b) if a>b then a else b end end
  module_function :abs, :less, :more
end
include  Util

require 'xlib/color_hsb'

  def best_color_match(depth, r, g, b)
    case depth
    when 16
      ((r & 0xf8) <<  8) | ((g & 0xfc) << 3) | ((b & 0xf8) >> 3)
    when 24, 32
      ((r & 0xff) << 16) | ((g & 0xff) << 8) |  (b & 0xff)
    else
      0
    end
  end

  def num2str(depth, c)
    case depth
    when 16
      lo = c & 0xff; c = c >> 8
      hi = c & 0xff; c = c >> 8
      hi.chr << lo.chr
    when 24, 32
      r = c & 0xff; c = c >> 8
      g = c & 0xff; c = c >> 8
      b = c & 0xff; c = c >> 8
      r.chr << g.chr << b.chr
    else
      "\001"
    end
  end

  def color_str(depth, r, g, b)
    num2str depth, best_color_match(depth, r, g, b)
  end

  def hsb_by_z(v)
    b = ( 75 - 180 * v.z / v.distance).to_i
    b = more b, 0
    b = less b, 255
    [0.2, 0.5, b/255.0]
  end

  def rgb_by_z(v)
    ColorHSB.hsb2rgb *hsb_by_z(v)
  end

  def color_by_z(v)
    Color.new *rgb_by_z(v)
  end

  def color_str_by_z(v)
    color_str @win_depth, *rgb_by_z(v)
  end

class ModelViewer
  Applet_area = Rect.new 0, 0, 300, 300
  Canvas_area = Rect.new 0, 0, 300, 300
  Center = Point.new
  Center.x = Canvas_area.x + Canvas_area.width  / 2
  Center.y = Canvas_area.y + Canvas_area.height / 2
  Canvas_color = $display.alloc_color 'white'

  def initialize
    @pre_x, @pre_y = Center.x, Center.y
    @sense = 0.01

    @nomal_inverse = false

    @win = Window.new $display.root, Applet_area
    @win.pixel = Canvas_color
    @gc = @win.new_gc

    @pix = @win.new_pixmap @width,@height
    @gc.window = @pix

    @win.event 'button_press' do |ev|
      draw_vertex
      @drag_x, @drag_y = ev.x, ev.y
      @drag = true
    end
  
    @win.event 'button_release' do |ev|
      @drag = false
      dx, dy = ev.x - @drag_x, ev.y - @drag_y
      phi, theta = dx * @sense, dy * @sense
      rotate phi, theta
      @drag_x, @drag_y = ev.x, ev.y
      draw_model
    end
  
    @win.event 'motion_notify' do |ev|
      if @drag
        dx, dy = ev.x - @drag_x, ev.y - @drag_y
        if dx**2+dy**2 >= 20**2
          phi, theta = dx * @sense, dy * @sense
          rotate phi, theta
          @drag_x, @drag_y = ev.x, ev.y
          draw_vertex
        end
      end
    end

    @win.event 'key_press' do |ev|
      case ev.keysym
      when ?z
        @nomal_inverse = ! @nomal_inverse
        p 'inverse'
      end
    end

    @win.event 'exposed' do paint end

    image_clear

    @win.title = $0
    @win.show

    @white_str = color_str @win.depth, 0xff, 0xff, 0xff
    @clean_map = @white_str * (Canvas_area.width * Canvas_area.height)
    @win_depth = @win.depth
  end

  def load_model_data(file_name)
    l = 0
    vn = 0
    fn = 0
    max_xyz = 0.001

    v_vec = []
    f_vec = []

    file = open file_name, 'r'
    while file.gets
      ary = split
      spec = ary.shift
      case spec
      when 'v'
        v = []; ary.each { |s| v.push s.to_f }
        max_xyz = more abs(v[0]), max_xyz
        max_xyz = more abs(v[1]), max_xyz
        max_xyz = more abs(v[2]), max_xyz
        v_vec.push v
      when 'f'
        f = []; ary.each { |s| f.push s.to_i }
        f_vec.push f
      end
    end
    file.close

    @vert = []
    @face = []
    v_vec.each { |v|
      @vert.push Vertex.new *v
    }
    f_vec.each { |f|
      @face.push Face.new @vert[f[0]-1], @vert[f[1]-1], @vert[f[2]-1]
    }
    @scale = 130/max_xyz

    @win.title = $0 +' - '+ file_name
  end

  def paint
    @pix.copy_area @win, @gc, Canvas_area
  end

  def image_clear
    @gc.fg = Canvas_color
    @gc.fill_rect Canvas_area
  end

  def draw_poly
    image_clear

    tmp = []
    @face.each { |f|
      a1 = f.v[1].x - f.v[0].x
      a2 = f.v[1].y - f.v[0].y
      a3 = f.v[1].z - f.v[0].z
      b1 = f.v[2].x - f.v[1].x
      b2 = f.v[2].y - f.v[1].y
      b3 = f.v[2].z - f.v[1].z

      f.n.x = a2 * b3 - a3 * b2
      f.n.y = a3 * b1 - a1 * b3
      f.n.z = a1 * b2 - a2 * b1

      if @nomal_inverse
        f.n.x = -f.n.x
        f.n.y = -f.n.y
        f.n.z = -f.n.z
      end

      if f.n.z < 0
        tmp.push f
        f.calc_depth
      end
    }

    tmp.sort!{ |a, b| b.depth <=> a.depth }
    tmp.each { |f|
      @gc.fg = $display.alloc_color(color_by_z f.n)
      p = []
      (0...3).each { |j|
        p.push(( f.v[j].x * @scale + Center.x).to_i)
        p.push((-f.v[j].y * @scale + Center.y).to_i)
      }
      @gc.fill_poly p, COORD_ORIG, POLY_FREE
    }
    $display.sync
  end

  def draw_vertex
    white = $display.alloc_color 'white'
    image = $display.new_image Canvas_area.width, Canvas_area.height
    image.fill white
    @vert.each { |v|
      x = (v.x * @scale + Center.x).to_i
      y = (v.y * @scale + Center.y).to_i
      p = $display.alloc_color(color_by_z v)
      image.put_pixel x, Canvas_area.height - y, p
    }
    @win.put_image image, @gc, Canvas_area
    $display.sync
  end

  def draw_model
    draw_poly
    paint
  end

  def rotate(phi, theta)
    sin, cos = Math.sin(theta), Math.cos(theta)
    @vert.each { |v| v.y, v.z =  v.y*cos + v.z*sin, -v.y*sin + v.z*cos }
    sin, cos = Math.sin(phi), Math.cos(phi)
    @vert.each { |v| v.x, v.z =  v.x*cos - v.z*sin,  v.x*sin + v.z*cos }
  end

end

class Face
  attr :v
  attr :n
  attr :depth

  def initialize(v0, v1, v2)
    @v = [v0, v1, v2]
    @n = Vertex.new 0, 0, 0
  end

  def calc_depth
    d = 0; @v.each { |v| d+= v.z }; @depth = d
  end
end

class Vertex
  attr :x, true
  attr :y, true
  attr :z, true

  def initialize(x, y, z)
    @x = x
    @y = y
    @z = z
  end

  def distance
    Math.sqrt @x ** 2 + @y ** 2 + @z ** 2
  end
end

def usage
  print "usage: ruby #$0 data_file_name\n"
  exit
end

file_name = ARGV.shift or usage

viewer = ModelViewer.new
viewer.load_model_data file_name
viewer.draw_model

while $display.event
end
