#!/usr/local/bin/ruby
require 'jcode'

class Foo
  TAB = '  '

  def initialize
    @tab = 0
    @col = TAB.length*@tab
    @maxcol = 40
    @buf = ""
    @align_stac = ['left']
  end

  def shift
    @tab+= 1
  end

  def unshift
    @tab-= 1
  end

  def align_push(align)
    @align_stac.push align
  end

  def align_pop
    @align_stac.pop
  end

  def align
    @align_stac[-1]
  end

  def line_feed
    tab = @tab
    tab+= 1 if @in_dd
    case align
    when 'left'
    when 'right'
      print ' '*(@maxcol-@buf.length)
    when 'center'
      print ' '*(@maxcol-@buf.length)>>1
    end
    print @buf,"\n",TAB*tab
    @col = TAB.length*tab
    @buf = ""
  end

  def draw_line
    tab = @tab
    tab+= 1 if @in_dd
    print '-'*(@maxcol-TAB.length*tab)
    print "\n",TAB*tab
    @col = TAB.length*tab
  end

  def put_flat(flat='')
    return if flat.empty?
    flat.split.each { |word|
      unless @buf.empty?
        @buf << ' '
        @col+=1
      end
      if word.length == word.jlength
        if @col + word.length > @maxcol
	  line_feed
        end
        @buf << word
        @col+= word.length
      else
	word.scan(/(.)/) { |char,|
	  if @col + char.length > @maxcol
	    line_feed
	  end
	  @buf << char
	  @col+= char.length
	}
      end
    }
  end

  def put_name(name,attr)
    @name = name
    print "[#@name]" if $DEBUG
    case name
    when 'div'
      line_feed
      align_push attr['align'] if attr['align']
    when '/div'
      line_feed
      align_pop
    when 'br'
      line_feed
    when 'hr'
      @in_dd = false
      line_feed
      draw_line
    when /h[1-6]/
      @in_dd = false
      line_feed
    when %r{/h[1-6]}
      line_feed
    when 'p'
      line_feed
    when '/p'
      line_feed
    when 'ul'
      shift
      line_feed
    when '/ul'
      unshift
      line_feed
    when 'ol'
      shift
      line_feed
    when '/ol'
      unshift
      line_feed
    when 'dl'
      shift
    when '/dl'
      unshift
    when 'dt'
      @in_dd = false
      line_feed
    when 'dd'
      @in_dd = true
      line_feed
    when 'li'
      line_feed
    end
  end
end

foo = Foo.new

filename = ARGV.shift || 'foo.html'
begin
  file = open filename, 'r'
  text = file.read
ensure
  file.close
end

$_ = text
while not (~/<([^>]+)>/).nil?
  flat = $`
  rest = $'
  tag = $1
  flat = flat.strip.gsub(/\n/,' ').gsub(/\s+/,' ')
  foo.put_flat flat
  if tag[0] == ?!
    $_ = rest
    next
  end
  name, *params = tag.split
  name.downcase!
  attr = {}
  params.each { |param|
    key, val = param.split('=')
    key.downcase!
    val = val.sub(/^"/,'').sub(/"$/,'') unless val.nil?
    attr[key] = val
  }
  foo.put_name name,attr
  $_ = rest
end
