#!/usr/local/bin/ruby
require 'jcode'
require 'xlib'
require 'util'
require 'drag'

Thread.abort_on_exception = true

POINT_ZERO = Xlib::Point.new

DIS = Xlib::Display.new
BGCOLOR = DIS.alloc_color 'white'
FGCOLOR = DIS.alloc_color 'black'
LINKCOLOR = DIS.alloc_color 'blue'

WIN = DragWindow.new DIS.root, 580, 400
WIN.pixel = BGCOLOR
WIN.show

font_name = %w(
  -adobe-utopia-*-*-*--12-*-*-*-p-*-iso8859-1
  -microsoft-gothic-*-*-*--12-*-*-*-c-*-jisx0208.1983-0
)
FONT = DIS.new_font font_name

require 'parser'
require 'designer'
require 'render'
require 'loader'

parser = Parser.new
designer = Designer.new
render = Render.new
loader = Loader.new

proc_born = proc { |text|
  parser.parse text
  designer.design parser.nodes
  render.rend designer.parts
  render.update
}

url = ARGV.shift || 'http://localhost/index.html'
parser.base = get_base(url)
text = loader.load url
proc_born.call text if text

@time = 1
@thr_rend = Thread.new {}
@loading = false

WIN.event 'button_press' do |ev|
  @time = ev.time
  Thread.kill @thr_rend if @thr_rend.status
@loading = true
  part = designer.point_mouse ev.x, ev.y, render.scry
  if part && part.respond_to?(:href)
    url = part.href
    begin
      text = loader.load url

      parser = Parser.new
      designer = Designer.new
      render = Render.new

      proc_born.call text if text
    rescue
      print $!, "\n"
      $@.each { |at| print at, "\n" }
    end
  end
@loading = false
end

=begin
@thr_hint = Thread.new {}

WIN.event 'motion_notify' do |x, y|
  Thread.kill @thr_hint if @thr_hint.status
@thr_hint = Thread.new {
  part = designer.point_mouse x, y, render.scry
  if part
    render.point_mouse part
  end
}
end
=end

@th = Thread.new {
  while true
    events = WIN.get_motion_events @time, 0
    unless events.empty?
      time, x, y = events[-1]
      part = designer.point_mouse x, y, render.scry
      if part
        render.point_mouse part
      end
    end
    Thread.pass
  end
}


WIN.drag do |x, y|
  unless @loading
    Thread.kill @thr_rend if @thr_rend.status
  @thr_rend = Thread.new {
    render.update render.scry + y
  }
  end
end

WIN.event 'exposed' do
  render.update
end

WIN.event 'delete' do
  exit
end

while DIS.event
end
