module Href
  attr :href, true
end

class Part
  def initialize
    @text = ""
    @area = Xlib::Rect.new
  end
  attr :text, true
  attr :area, true
end

class Designer
  HTAB = 56
  def initialize
    @parts = []
    @in_line = nil
  end
  attr :parts

  def point_mouse(x, y, scry=0)
    @parts.each { |part|
      r = part.area
      if r.x < x && x < r.x+r.width && r.y < y+scry && y+scry < r.y+r.height
        return part
      end
    }
    nil
  end

  def design(nodes)
    p "design start"
    y = 20
    x = 8
    x_init = x
    x_min = x
    x_base = x
    x_max = WIN.width - 8 # x + HTAB * 10
    nodes.each { |node|
      ink, logical = FONT.extents("$B9H(B")
      case node
      when Text
        text = node.text
        ink, logical = FONT.extents(text)
        while x + logical.width > x_max
          cut, i, line = 0, 0, ""
          text.scan(/(.)/) { |char,|
            line << char
            esc = FONT.escapement(line)
            if x + esc > x_max
              cut = i
              break
            end
            i+= char.size
          }
          part = Part.new
          part.text = text[0...cut]
          part.area.x = x
          part.area.y = y
          part.area.width  = logical.width
          part.area.height = logical.height
          @parts.push part
          if node.respond_to? :href
            part.extend Href
            part.href = node.href
          end
          text = text[cut...text.size]
          ink, logical = FONT.extents(text)
          x = x_base
          y+= logical.height
        end
        part = Part.new
        part.text = text
        part.area.x = x
        part.area.y = y
        part.area.width  = logical.width
        part.area.height = logical.height
        @parts.push part
        if node.respond_to? :href
          part.extend Href
          part.href = node.href
        end
        x+= logical.width
      when Sign
        if @in_line
          if @in_line != x
            y+= logical.height
          end
          @in_line = nil
        end
        case node.sign
        when 'br'
          x = x_base
          y+= logical.height
        when 'ul'
          x_min+= HTAB
          x_base+= HTAB
          x = x_base
        when '/ul'
          x_min-= HTAB
          x_base-= HTAB
          x = x_base
        when 'li'
          x = x_base
          @in_line = x
        when 'dl'
          x_min+= HTAB
          x_base = x_min
          x = x_base
        when '/dl'
          x_min-= HTAB
          x_base = x_min
          x = x_base
        when 'dt'
          x_base = x_min
          x = x_base
          @in_line = x
        when 'dd'
          x_base = x_min + HTAB
          x = x_base
          @in_line = x
        end
      end
    }
    p "design done."
  end
end

