#!/usr/local/bin/ruby

SEP = "\r\n"
TIMEOUT = 30

require 'socket'
require 'timeout'
require 'dnload'

#      url ::= proto://serv[:port][/[dirs]]
# mini url ::=         serv[:port][/[dirs]]

def parse_url(url)
  if %r!^([a-z]+)://! =~ url
    # protocol specified
    proto = $1
    mini  = $'
#   if proto == "http"
#     # for http
#   else
#     raise "error: #{proto} is unsupported protocol\n"
#   end
  else
    proto = "http"
    mini  = url
  end
  
  serv, dirs = mini.scan(%r!([^/]+)(.*)!)[0]
  serv, port = serv.scan(%r!([^:]+)(.*)!)[0]
  port = port.to_i
  port = 80 if port == 0
  
  dirs = "/" if dirs == ""
  if dirs =~ %r!/$!
#   file = "/"
  else
    dir_ary = dirs.split(%r!/!)
    file = dir_ary.pop
    if file =~ /#/
      file, name = file.split(/#/)
    end
    dirs = dir_ary.join("/") + "/"
  end
  results = [proto, serv, port, dirs, file, name]
# print "URL: ", results.join(" "), "\n"
  results
end

def creep(note, url)
print "creep ...\n"
  proto, serv, port, dirs, file, name = parse_url(url)
  timeout(TIMEOUT) {
  begin
    sock = TCPsocket.open(serv, port)
    sock.print "GET #{dirs}#{file} HTTP/1.0\r\nAccept: */*\r\n\r\n"
    head = sock.gets("\r\n\r\n")
    sock.close
    
    status = head.split(SEP).shift
    %r!^(\w+)/([\d.]+)\s+(\d+)\s*! =~ status
    http, version, code, message = $1, $2, $3, $'
    code = code.to_i

    if code == 200
      dnload(url)
    else
      raise "error: code=#{code}\n"
    end
    
    print %!#{note} #{url} (#{code} #{message})\n!
  rescue
    print %!#{note} #{url} (#$!)\n!
  end
  }
print "creep done.\n"
end

def web_get_body(url)
print "web get body ...\n"
  proto, serv, port, dirs, file, name = parse_url(url)
  body = nil
  timeout(TIMEOUT) {
    sock = TCPsocket.open(serv, port)
    sock.print "GET #{dirs}#{file} HTTP/1.0\r\nAccept: */*\r\n\r\n"
    head = sock.gets("\r\n\r\n")
    len = if /^Content-length:\s*(\S+)/i =~ head then $1.to_i else nil end
    body = if len then sock.read(len) else sock.read end
    sock.close

    status = head.split(SEP).shift
    %r!^(\w+)/([\d.]+)\s+(\d+)\s*! =~ status
    http, version, code, message = $1, $2, $3, $'
    code = code.to_i

    if code == 200
      dnload(url)
    else
      raise "error: code=#{code}\n"
    end
  }
print "web get body done.\n"
  body
end

def get_web_body_with_cache(url)
  proto, serv, port, dirs, file, name = parse_url(url)
  # serach local cache
  dir_ary = dirs.split("/")
  dir_ary[0] = serv
  dir_ary.unshift 'web'
  chdirs = []
  dir_p = dir_ary.each { |dir|
    if dir[0] == ?~
       dir[0] =  ?_
    end
    if FileTest.exist?(dir)
      Dir.chdir dir
      chdirs.push dir
      print "cd #{dir}\n"
    else
      break
    end
  }
  cache_p = dir_p && FileTest.exist?(file)
  chdirs.each { |dir| Dir.chdir '..' }
  if cache_p
    print "local cache found.\n"
    body = open(dir_ary.join("/")+"/"+file).read
  else
    print "local cache not found.\n"
    body = web_get_body(url)
  end
  return body, cache_p
end

def get_base(url)
print "get base\n"
  proto, serv, port, dirs, file, name = parse_url(url)
  "http://#{serv}#{dirs}"
end

def base_href_expand(base, href)
  proto, serv, port, dirs, file, name = parse_url(base)
  if %r!^([a-z]+)://! =~ href
    # for protocol
    href
  else
    if %r!/$! =~ base
      if %r!^/! =~ href
        "#{proto}://#{serv}#{href}"
      else
        "#{base}#{href}"
      end
    else
      if %r!^/! =~ href
        "#{base}#{href}"
      else
        "#{proto}://#{href}"
      end
    end
  end
end
