
module Anchor
  attr :name, true
  attr :href, true
end

class Node
end

class Text < Node
  attr :text, true
  def initalize
    @text = ""
  end
end

class Sign < Node
  attr :sign, true
  def initalize
    @sign = ""
  end
end

class Parser
  def initialize
    @nodes = []
    @in_title = false
    @in_pre = false
    @attr_base = nil
    @attr_a = nil
    @base = "http://localhost/"
  end
  attr :nodes
  attr :base, true

  def close_marker(mark)
    effect, *params = mark.split
    effect.downcase!
    case effect
    when 'title'
      @in_title = false
    when 'pre'
      @in_pre = false
    when 'a'
      @attr_a = nil
    when 'ul'
      node = Sign.new
      node.sign = '/ul'
      nodes.push node
    when 'dl'
      node = Sign.new
      node.sign = '/dl'
      nodes.push node
    end
  end

  def open_marker(mark)
    effect, *params = mark.split
    effect.downcase!
    attr = {}
    params.each { |param|
      key, val = param.scan(/([^=]+)=?(.*)/)[0]
      key.downcase!
      val = val.sub(/^"/,'').sub(/"$/,'') unless val.nil?
      attr[key] = val
    }
    case effect
    when 'title'
      @in_title = true
    when 'pre'
      @in_pre = true
    when 'base'
      @attr_base = attr
    when 'a'
      @attr_a = attr
    when 'br'
      node = Sign.new
      node.sign = 'br'
      nodes.push node
    when 'ul'
      node = Sign.new
      node.sign = 'ul'
      nodes.push node
    when 'li'
      node = Sign.new
      node.sign = 'li'
      nodes.push node
    when 'dl'
      node = Sign.new
      node.sign = 'dl'
      nodes.push node
    when 'dt'
      node = Sign.new
      node.sign = 'dt'
      nodes.push node
    when 'dd'
      node = Sign.new
      node.sign = 'dd'
      nodes.push node
    end
  end

  def put_marker(mark)
    if %r!^/! =~ mark
      close_marker $'
    else
      open_marker mark
    end
  end

  def put_string(s)
    if @in_title
      WIN.title = s
      return
    end
    s.gsub!(/&quot;/i, '"')
    s.gsub!(/&lt;/i, "<")
    s.gsub!(/&gt;/i, ">")
    s.gsub!(/&amp;/i, "&")
    if @in_pre
      ary = s.split(/\e/)
      ary.each { |i|
        node = Text.new
        node.text = i
        @nodes.push node

        node = Sign.new
        node.sign = 'br'
        nodes.push node
      }
      return
    end
    s.gsub!(/\e+/, " ")
    s.gsub!(/\s+/, " ")
    node = Text.new
    node.text = s
    if @attr_a
      node.extend Anchor
      node.name = @attr_a['name']
      node.href = base_href_expand(if @attr_base then @attr_base['href'] else @base end, @attr_a['href'])
    end
    @nodes.push node
  end

  def parse(text)
    p 'parse start'
    text.gsub!(/\n/, "\e")
    list = text.split(/(<[^>]+>)/)
    list.each { |s|
      case s
      when /^$/
      when /^<(.+)>$/
        put_marker $1
      else
        put_string s
      end
    }
    p 'parse done.'
  end
end

