/**
 * visual.c
 *
 * by YOSHIDA Kazuhiro (moriq.kazuhiro@nifty.ne.jp)
 */

#include "ruby.h"
#include "init.h"

#include "visual.h"

static char *color_class_name[] = {
	"StaticGray",
	"GrayScale",
	"StaticColor",
	"PseudoColor",
	"TrueColor",
	"DirectColor"
};

void
xlib_visual_free(visp)
	t_visual *visp;
{
	XFree(visp);
}

static VALUE
xlib_visual_equal(src_a, src_b)
	VALUE src_a, src_b;
{
	t_visual *a, *b;
	Data_Get_Struct(src_a, t_visual, a);
	Data_Get_Struct(src_b, t_visual, b);
	return (a->visualid == b->visualid) ? Qtrue : Qfalse;
}

static VALUE
xlib_visual_get_depth(self)
	VALUE self;
{
	t_visual *visp;
	Data_Get_Struct(self, t_visual, visp);
	return INT2NUM(visp->depth);
}

static VALUE
xlib_visual_get_color_class(self)
	VALUE self;
{
	t_visual *visp;
	Data_Get_Struct(self, t_visual, visp);
	return rb_str_new2(color_class_name[visp->class]);
}

static VALUE
xlib_visual_to_s(self)
	VALUE self;
{
	char *s;
	char *cname = rb_class2name(CLASS_OF(self));
	t_visual *visp;
	VALUE str;

	s = ALLOCA_N(char, 256);
	Data_Get_Struct(self, t_visual, visp);
	sprintf(s, "#<%s depth=%d color_class=%s>", cname,
		visp->depth,
		color_class_name[visp->class]
	);
	str = rb_str_new2(s);

	return str;
}

void
xlib_init_visual()
{
	cXlib_visual = rb_define_class_under(mXlib, "Visual", rb_cObject);
	rb_define_method(cXlib_visual, "==", xlib_visual_equal, 1);
	DefineMethodGet(visual, depth);
	DefineMethodGet(visual, color_class);
	DefineMethod(visual, to_s, 0);
}
